from Acquisition import aq_inner
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.tagging.behavior import ITagging
from ftw.tagging.interfaces.tagging import ITagRoot
from ftw.tagging.utils import getInterfaceRoot
from plone.app.layout.viewlets.common import ViewletBase


class TagListViewlet(ViewletBase):
    index = ViewPageTemplateFile('taglist.pt')

    def update(self):
        context = aq_inner(self.context).aq_explicit

        behavior = ITagging(context, None)
        if behavior:
            self.tags = behavior.tags

        self.tag_root_url = getInterfaceRoot(context, ITagRoot).absolute_url()



from zope.i18nmessageid import MessageFactory

taggingMessageFactory = MessageFactory('ftw.tagging')

_ = taggingMessageFactory


def initialize(context):
    """Initializer called when used as a Zope 2 product."""



from ftw.tagging import utils
from ftw.tagging.interfaces.tagging import ITaggable, ITagRoot
from ftw.testing import MockTestCase


class TestUtils(MockTestCase):

    def setUp(self):
        super(TestUtils, self).setUp()
        portal_url = self.stub()
        self.mock_tool(portal_url, 'portal_url')
        self.portal = self.stub()
        self.expect(portal_url.getPortalObject()).result(self.portal)

    def test_getInterfaceRoot(self):
        root = self.providing_stub([ITagRoot])

        obj = self.set_parent(
            self.stub(),
            self.set_parent(
                self.stub(),
                self.set_parent(
                    root,
                    self.portal)))

        self.replay()

        self.assertEqual(utils.getInterfaceRoot(obj, ITagRoot), root)

    def test_getInterfaceRoot_when_root_reached(self):
        obj = self.set_parent(
            self.stub(),
            self.portal)

        self.replay()

        self.assertEqual(utils.getInterfaceRoot(obj, ITagRoot), self.portal)

    def test_getTagRootTags(self):
        brain1 = self.stub()
        self.expect(brain1.tags).result(('foo', 'bar', u'\xe4', '\xc3\xa4'))
        brain2 = self.stub()
        self.expect(brain2.tags).result(('bar', u'baz'))
        brain3 = self.stub()
        self.expect(brain3.tags).result(None)

        root = self.providing_stub([ITagRoot])
        obj = self.set_parent(self.stub(), root)
        self.expect(root.getPhysicalPath()).result(['', 'path', 'to', 'root'])

        catalog = self.stub()
        self.mock_tool(catalog, 'portal_catalog')
        query = {'path': '/path/to/root',
                 'object_provides': ITaggable.__identifier__}
        self.expect(catalog(query)).result([brain1, brain2])

        self.replay()

        self.assertEqual(set(utils.getTagRootTags(obj)),
                         set(['foo', 'bar', 'baz', '\xc3\xa4']))



from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.tagging.utils import getBrainsByTag


class ObjectsByTagView(BrowserView):
    template = ViewPageTemplateFile('objects_by_tag.pt')

    def __call__(self):
        context = aq_inner(self.context).aq_explicit
        request = self.request

        self.brains = []
        tag = getattr(request, 'tag', None)
        if tag is not None:
            self.brains = getBrainsByTag(context, tag)

        return self.template()



from ftw.testing import IS_PLONE_5
from ftw.testing.genericsetup import GenericSetupUninstallMixin
from ftw.testing.genericsetup import apply_generic_setup_layer
from unittest import TestCase
from unittest import skipUnless


@apply_generic_setup_layer
@skipUnless(IS_PLONE_5, 'Test the uninstall profile for Plone 5')
class TestGenericSetupUninstall(TestCase, GenericSetupUninstallMixin):
    package = 'ftw.tagging'
    skip_files = ('viewlets.xml',)



from Acquisition import aq_base
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import utils
from ftw.keywordwidget.utils import safe_utf8
from ftw.tagging.interfaces.tagging import ITaggable, ITagRoot


def getInterfaceRoot(context, interface):
    """Climb up the folder hierarchy and
       find an object that implements ISearchRoot
    """

    portal_url = getToolByName(context, 'portal_url')

    portal = portal_url.getPortalObject()
    object = context

    while not interface.providedBy(object) and \
            aq_base(object) is not aq_base(portal):
        object = utils.parent(object)
    if interface.providedBy(object) and \
            aq_base(object) is not aq_base(portal):
        return object
    else:
        return aq_base(portal)


def getTagRootTags(context):
    """Get all tags used in the current branch of the tag root
    """

    items = set()
    catalog_tool = getToolByName(context, "portal_catalog")
    tag_root = getInterfaceRoot(context, ITagRoot)
    root_path = '/'.join(tag_root.getPhysicalPath())

    brains_below_tag_root = catalog_tool({
                                'path': root_path,
                                'object_provides': ITaggable.__identifier__})

    for brain in brains_below_tag_root:
        tags = brain.tags
        if not isinstance(tags, (tuple, list)):
            continue
        for tag in tags:
            items.add(safe_utf8(tag))

    return list(items)


def getBrainsByTag(context, tag):
    """Get catalog brains for objects in the current branch
       of the tag root by tag
    """

    catalog_tool = getToolByName(context, "portal_catalog")
    tag_root = getInterfaceRoot(context, ITagRoot)
    root_path ='/'.join(tag_root.getPhysicalPath())

    brains = catalog_tool({'path': root_path,
                            'object_provides': ITaggable.__identifier__,
                            'tags': safe_utf8(tag)})

    return brains



from ftw.keywordwidget.field import ChoicePlus
from ftw.keywordwidget.widget import KeywordFieldWidget
from ftw.tagging import _
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from z3c.form.interfaces import IAddForm, IEditForm
from zope import schema
from zope.interface import alsoProvides


class ITagging(model.Schema):

    model.fieldset(
        'tags',
        label=_(u'label_schema_tags', default=u'Tags'),
        fields=[
            'tags',
        ],
    )

    directives.widget('tags', KeywordFieldWidget)
    tags = schema.List(
        title=_(u'label_tags', default=u'Tags'),
        value_type=ChoicePlus(
            title=u'Multiple',
            vocabulary='tags',
        ),
        required=False,
        missing_value=(),
    )

    # Show field only on edit/add forms.
    directives.omitted('tags', )
    directives.no_omit(IEditForm, 'tags', )
    directives.no_omit(IAddForm, 'tags', )


alsoProvides(ITagging, IFormFieldProvider)



from ftw.builder import builder_registry
from ftw.builder.portlets import PlonePortletBuilder
from ftw.tagging.portlets import tags


class TagCloudPortletBuilder(PlonePortletBuilder):
    assignment_class = tags.Assignment

builder_registry.register('tag cloud portlet', TagCloudPortletBuilder)



from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.tagging import taggingMessageFactory as _
from ftw.tagging.interfaces.tagging import ITaggable, ITagRoot
from ftw.tagging.utils import getInterfaceRoot, getTagRootTags
from plone.app.portlets.portlets import base
from plone.dexterity.utils import safe_utf8
from plone.portlets.interfaces import IPortletDataProvider
from zope import schema
from zope.interface import implements
from z3c.form import field


try:
    from plone.app.portlets.browser import z3cformhelper
except ImportError:
    # plone 5
    from plone.app.portlets.browser import formhelper as z3cformhelper


class ITagsPortlet(IPortletDataProvider):

    maxsize = schema.ASCIILine(title=_(u'Max. Fontsize'),
                               description=_(u'Size in em'),
                               required=True,
                               default='2')

    minsize = schema.ASCIILine(title=_(u'Min. Fontsize'),
                               description=_(u'Size in em'),
                               required=True,
                               default='0.7')


class Assignment(base.Assignment):
    implements(ITagsPortlet)

    def __init__(self, maxsize='2', minsize='0.7'):
        self.maxsize = maxsize
        self.minsize = minsize

    @property
    def title(self):
        return "Tag Cloud Portlet"


class Renderer(base.Renderer):

    def __init__(self, context, request, view, manager, data):
        self.context = context
        self.data = data
        self.request = request

        tags = getTagRootTags(context)
        tag_root = getInterfaceRoot(context, ITagRoot)
        root_path ='/'.join(tag_root.getPhysicalPath())

        catalog_tool = getToolByName(context, "portal_catalog")
        query = {}
        query['object_provides'] = ITaggable.__identifier__
        query['path'] = root_path

        tag_occurrence = {}
        for tag in tags:
            query['tags'] = safe_utf8(tag)
            tag_occurrence[tag] = len(catalog_tool(query))

        weight_list = tag_occurrence.values()
        weight_list.sort()

        if weight_list:
            minimal = weight_list[:1][0]
            maximal = weight_list[-1:][0]

            maxsize = float(self.data.maxsize)
            minsize = float(self.data.minsize)

            tag_cloud = []

            for tag in tags:
                try:
                    size = float((maxsize * \
                                      (tag_occurrence[tag] - minimal))) / \
                                      float((maximal - minimal))
                except ZeroDivisionError:
                    size = 1
                if tag_occurrence[tag] <= minimal or size < minsize:
                    size = float(self.data.minsize)

                info = dict(title=tag,
                            font_size=round(size, 1))
                tag_cloud.append(info)

            tag_cloud.sort(lambda x, y: cmp(x['title'].lower(),
                                            y['title'].lower()))
            self.tag_cloud = tag_cloud
        else:
            self.tag_cloud = []

        self.tag_root_url = tag_root.absolute_url()

    @property
    def available(self):
        """only show the portlet,

        when already tags are defined in this tagroot"""

        if len(self.tag_cloud) == 0:
            return False
        else:
            return True

    render = ViewPageTemplateFile('tags.pt')


class AddForm(z3cformhelper.AddForm):
    schema = ITagsPortlet
    fields = field.Fields(ITagsPortlet)
    label = _(u"Add Tag Cloud Portlet")
    description = _(u"This portlet displays a Tag Cloud \
                      for Tags within the current Tag Root.")

    def create(self, data):
        return Assignment(maxsize=data.get('maxsize', '2'),
                          minsize=data.get('minsize', '0.7'))


class EditForm(z3cformhelper.EditForm):
    schema = ITagsPortlet
    fields = field.Fields(ITagsPortlet)
    label = _(u"Edit Tag Cloud Portlet")
    description = _(u"This portlet displays a Tag Cloud\
                      for Tags within the current Tag Root.")



from binascii import b2a_qp
from ftw.tagging.utils import getTagRootTags
from Products.CMFPlone.utils import safe_unicode
from zope.interface import alsoProvides
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


def tagVocabulary(context):
    """Vocabulary factory for tags within a given section of the site,
    delimited by an ITagRoot interface

    Inspired by "plone.app.vocabularies.catalog.KeywordsVocabulary" for the
    encoding stuff.
    """

    root_tags = getTagRootTags(context)

    def safe_encode(term):
        if isinstance(term, unicode):
            # no need to use portal encoding for transitional encoding from
            # unicode to ascii. utf-8 should be fine.
            term = term.encode('utf-8')
        return term

    # Vocabulary term tokens *must* be 7 bit values, term titles *must* be unicode.
    items = [
        SimpleTerm(tag, b2a_qp(safe_encode(tag)), safe_unicode(tag))
        for tag in root_tags
    ]
    return SimpleVocabulary(items)


alsoProvides(tagVocabulary, IVocabularyFactory)



from plone import api
import transaction
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory


def add_behaviors(type_to_configure, *additional_behaviors):
    fti = api.portal.get().portal_types.get(type_to_configure)
    behaviors = list(fti.behaviors)
    behaviors += list(additional_behaviors)
    fti.behaviors = tuple(set(behaviors))
    transaction.commit()


def terms_for(vocabulary_name, context):
    factory = getUtility(IVocabularyFactory,
                         name=vocabulary_name)
    return dict([(term.value, term.title)
                 for term in factory(context)])



# -*- coding: utf-8 -*-
from ftw.builder import Builder
from ftw.builder import create
from ftw.builder.content import register_dx_content_builders
from ftw.tagging.tests import FunctionalTestCase
from ftw.tagging.tests import utils
from ftw.tagging.utils import getTagRootTags
from ftw.testbrowser import browsing
from plone import api
from plone.app.testing import applyProfile
from ftw.testing import IS_PLONE_5


class TestTagsDexterity(FunctionalTestCase):

    def setUp(self):
        super(TestTagsDexterity, self).setUp()
        self.grant('Manager')
        applyProfile(self.portal, 'plone.app.contenttypes:default')
        register_dx_content_builders(force=True)

    def test_tags_indexer(self):
        utils.add_behaviors('Document', 'ftw.tagging.behavior.ITagging')

        document = create(Builder('document')
                          .titled(u'My Document')
                          .having(tags=['Foo', u'Bär']))

        catalog = api.portal.get_tool('portal_catalog')
        rid = catalog.getrid('/'.join(document.getPhysicalPath()))
        index_data = catalog.getIndexDataForRID(rid)

        # We expect UTF8 byte strings form the catalog index.
        self.assertEqual(
            {'Foo', 'B\xc3\xa4r'},
            set(index_data['tags'])
        )

        # We expect UTF8 byte strings form the catalog index.
        self.assertEqual(
            {'Foo', 'B\xc3\xa4r'},
            set(getTagRootTags(document))
        )

    @browsing
    def test_tags_widget(self, browser):
        utils.add_behaviors('Document', 'ftw.tagging.behavior.ITagging')

        document = create(Builder('document')
                          .titled(u'My Document'))

        browser.login().visit(document, view='edit')

        # Set two tags "Foo" and "Bär" on the document.
        browser.fill({'form.widgets.ITagging.tags_new': u'Foo\nBär'}).save()

        catalog = api.portal.get_tool('portal_catalog')
        rid = catalog.getrid('/'.join(document.getPhysicalPath()))
        index_data = catalog.getIndexDataForRID(rid)

        self.assertEqual(
            {'B\xc3\xa4r', 'Foo'},
            set(index_data['tags'])
        )
        self.assertEqual(
            {'B\xc3\xa4r', 'Foo'},
            set(getTagRootTags(document))
        )

    def test_plone_is_tag_root(self):
        utils.add_behaviors('Document', 'ftw.tagging.behavior.ITagging')

        root1 = create(Builder('folder').titled(u'Tag Root 1'))
        document1 = create(Builder('document')
                           .titled(u'Document in Tag Root 1')
                           .within(root1)
                           .having(tags=['Foo']))

        root2 = create(Builder('folder').titled(u'Tag Root 2'))
        document2 = create(Builder('document')
                           .titled(u'Document in Tag Root 2')
                           .within(root2)
                           .having(tags=['Bar']))

        self.assertEqual(
            {'Foo', 'Bar'},
            set(getTagRootTags(document1))
        )
        self.assertEqual(
            {'Foo', 'Bar'},
            set(getTagRootTags(document2))
        )

    def test_folders_are_tag_roots(self):
        utils.add_behaviors('Folder', 'ftw.tagging.interfaces.tagging.ITagRoot')
        utils.add_behaviors('Document', 'ftw.tagging.behavior.ITagging')

        root1 = create(Builder('folder').titled(u'Tag Root 1'))
        document1 = create(Builder('document')
                           .titled(u'Document in Tag Root 1')
                           .within(root1)
                           .having(tags=['Foo']))

        root2 = create(Builder('folder').titled(u'Tag Root 2'))
        document2 = create(Builder('document')
                           .titled(u'Document in Tag Root 2')
                           .within(root2)
                           .having(tags=['Bar']))

        self.assertEqual(
            ['Foo'],
            getTagRootTags(document1)
        )
        self.assertEqual(
            ['Bar'],
            getTagRootTags(document2)
        )

    @browsing
    def test_objects_by_tag_view(self, browser):
        utils.add_behaviors('Folder', 'ftw.tagging.interfaces.tagging.ITagRoot')
        utils.add_behaviors('Document', 'ftw.tagging.behavior.ITagging')

        folder = create(Builder('folder'))

        document1 = create(Builder('document')
                           .titled(u'Document 1')
                           .within(folder)
                           .having(tags=['Foo']))

        document2 = create(Builder('document')
                           .titled(u'Document 2')
                           .within(folder)
                           .having(tags=['Foo', 'Bar']))

        browser.login().visit(document1)
        browser.click_on('Foo')

        # We're on the "@@objects_by_tag_view" now.
        self.assertEqual(
            'http://nohost/plone/folder/@@objects_by_tag_view?tag=Foo',
            browser.url
        )
        # Both documents are listed because the have the same tag "Foo".
        self.assertEqual(
            ['Document 1', 'Document 2'],
            browser.css('#content h2').text
        )

        # Go to the detail view of the listed objects.
        browser.click_on('Document 2')
        self.assertEqual(
            document2.absolute_url(),
            browser.url
        )

    @browsing
    def test_tag_cloud_portlet(self, browser):
        utils.add_behaviors('Document', 'ftw.tagging.behavior.ITagging')

        create(Builder('document')
               .titled(u'My Document')
               .having(tags=['Foo', u'Bär']))

        create(Builder('tag cloud portlet'))

        browser.login().open()
        if IS_PLONE_5:
            css_selector = '.portletTagcloud .portletContent a'
        else:
            css_selector = '.portletTagcloud .portletItem a'

        self.assertEqual(
            [u'B\xe4r', 'Foo'],
            browser.css(css_selector).text
        )



from zope.interface import Interface


class ITaggable(Interface):
    """Marker interface for types that can be tagged
    """


class ITagRoot(Interface):
    """Marker interface that delimits ranges where tags are valid
    """



from ftw.tagging.behavior import ITagging
from ftw.tagging.interfaces.tagging import ITaggable
from plone.dexterity.utils import safe_utf8
from plone.indexer import indexer
from zope.interface import Interface


@indexer(Interface)
def tags(obj):
    if not ITaggable(obj, None):
        return None

    behavior = ITagging(obj, None)
    if behavior:
        return map(safe_utf8, behavior.tags)

    return None
